@@ -36,7 +36,7 @@ def get_software() -> str:
 
 class KeyLock:
     def __init__(self) -> None:
-        self._locks: "Final[WeakValueDictionary[Hashable," "asyncio.Lock]]" = (
+        self._locks: Final[WeakValueDictionary[Hashable, asyncio.Lock]] = (
             WeakValueDictionary()
         )
 
